#include<iostream>
#include<string>
#include<vector>
#include<deque>
#include<map>
using namespace std;


struct AhoTrie {
	struct Node {
		map<char, Node*> next;
		Node* link, *parent;
		vector<Node*> linking;
		char lastChar;
		int count = 0;
	};
	deque<Node> nodes;
	Node* root;
	
	AhoTrie() {
		nodes.resize(1);
		root = &nodes[0];
	}
	
	Node* addString(string s) {
		Node* cur = root;
		for(char c : s) {
			if(!cur->next.count(c)) {
				nodes.push_back(Node());
				cur->next[c] = &nodes.back();
				nodes.back().parent = cur;
				nodes.back().lastChar = c;
			}
			cur = cur->next[c];
		}
		return cur;
	}
	
	void initializeLinks() {
		vector<Node*> front(1, root);
		for(int i=0;i<(int)front.size();i++) {
			Node* cur = front[i];
			for(pair<char, Node*> cp : cur->next)
				front.push_back(cp.second);
				
			if(cur == root || cur->parent == root) {
				cur->link = root;
				continue;
			}
			cur->link = cur->parent->link;
			while(cur->link != root && !cur->link->next.count(cur->lastChar) )
				cur->link = cur->link->link;
			if(cur->link->next.count(cur->lastChar) )
				cur->link = cur->link->next[cur->lastChar];
		}
	}
	
	void addAppearances(string s) {
		Node* cur = root;
		for(auto c : s) {
			while(cur != root && !cur->next.count(c) )
				cur = cur->link;
			if(cur->next.count(c))
				cur = cur->next[c];
			cur->count++;
		}
		
		vector<Node*> front(1, root);
		for(int i=0;i<(int)front.size();i++)
			for(pair<int, Node*> cp : front[i]->next)
				front.push_back(cp.second);
		
		for(int i=front.size()-1; i>0; i--)
			front[i]->link->count += front[i]->count;
	}
};


int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	
	AhoTrie trie;
	int n, m;
	cin >> n >> m;
	string a;
	cin >> a;
	
	vector<AhoTrie::Node*> askedNodes;
	for(int i=0;i<m;i++) {
		int len;
		string s;
		cin >> len >> s;
		askedNodes.push_back(trie.addString(s) );
	}
	trie.initializeLinks();
	trie.addAppearances(a);
	
	for(auto cur : askedNodes)
		cout << cur->count << '\n';
	return 0;
}

